# pptable - pretty print tables in Python

# pptable is a rewrite of Ryan Ginstrom's padnums module,
# available at
# http://ginstrom.com/scribbles/2007/09/04/pretty-printing-a-table-in-python/.
# In comparison to padnums, pptable is a bit more flexible and
# efficient.

# Copyright (c) 2010--2011 Claude Rubinson and Ryan Ginstrom

# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# constants
l = 'l'
r = 'r'

def _format(field, format=None):
    """Format number using printf syntax or convert to string."""

    if not format:
        format = '%s'

    if isinstance(field, (bool,str)):
        return str(field)
    else:
        try:
            return format % field
        except (TypeError, ValueError), exc:
            msg = 'invalid string formatting specification: %s' % format
            exc.args = (msg,)
            raise

def pp(table, template=None, delim=' '):
    """Pretty print a table, left/right padding for alignment.

    table is a rectangular list of lists, with each row having the
    same number of columns.

    template is a comma-delimited string where elements are of the
    form 'alignment[%printf]'; that is, a prefix 'l' or 'r',
    indicating whether to left or right justify that column,
    optionally followed by a printf formatting command.  If template
    is unspecified, the first column will be left justified and all
    other columns will be right justified.

    delim is the output delimiter; default is a single space."""

    # default template has first column left justified and remaining
    # columns right justified
    if not template:
        fmt = [(l,None)] + [(r,None)] * (len(table[0])-1)
    # fmt is a list of tuples, indicating whether to left or right
    # justify and specifying number formatting
    else:
        fmt = []
        for col in template.split(','):
            fmt.append((col[0], col[1:]))

    # get max width of each column
    column_widths = []
    for col in range(len(table[0])):
        column_widths.append(max([len(_format(row[col],fmt[col][1])) for row in table]))

    out = ''
    for row in table:
        for i, col in enumerate(row):
            if fmt[i][0] == l:
                out += _format(col,fmt[i][1]).ljust(column_widths[i]) + delim
            elif fmt[i][0] == r:
                out += _format(col,fmt[i][1]).rjust(column_widths[i]) + delim
            else:
                msg = 'invalid formatting specification: %s' % fmt[i][0]
                raise Exception, msg
        out = out.rstrip(delim)
        out += '\n'
    return(out.rstrip('\n'))

if __name__=="__main__":
    table = [["", "taste", "life", "land speed"],
             ["spam", 300101, 4, 1003],
             ["eggs", 105, 3.00, 42],
             ["lumberjacks", 13, False, 10]]

    print pp(table)
